unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    DACRadios: TRadioGroup;
    LevelRadios: TRadioGroup;
    CalibratePanel: TPanel;
    CalGroupHi: TGroupBox;
    CalGroupLo: TGroupBox;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    CalCountEditHi: TEdit;
    mVErrorEditHi: TEdit;
    mVErrorButtonHi: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Button6: TButton;
    Button7: TButton;
    Button8: TButton;
    Button9: TButton;
    CalCountEditLo: TEdit;
    mVErrorEditLo: TEdit;
    mVErrorButtonLo: TButton;
    Label3: TLabel;
    Label4: TLabel;
    BlurbMemo: TMemo;
    Panel1: TPanel;
    WriteAllButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure WriteOneButtonClick(Sender: TObject);
    procedure WriteAllButtonClick(Sender: TObject);
    procedure DACRadiosClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure LevelRadiosClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure mVErrorButtonClick(Sender: TObject);
    procedure AdjustButtonClick(Sender: TObject);
    procedure CalCountEditChange(Sender: TObject);
    procedure CalCountEditExit(Sender: TObject);
    procedure CalCountEditKeyPress(Sender: TObject; var Key: Char);
    procedure mVErrorEditKeyPress(Sender: TObject; var Key: Char);
  protected
    CalCountEdit, mVErrorEdit: array[0..1] of TEdit;
    mVErrorButton: array[0..1] of TButton;

    procedure IdleInit(Sender: TObject; var Done: Boolean);
    procedure CheckAllButtonClick(Sender: TObject);
    procedure UncheckAllButtonClick(Sender: TObject);
    procedure BuildCalFile(iDAC: Integer);
    procedure UpdateOutput(iDAC: Integer);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math,
  AIOUSB,
  TheseUtils,
  DetectUnit, WaitUnit;

var
  DeviceIndex: LongWord;
  Channels: Integer;
  ChMask: LongWord;
  CalFN: String;
  CalTar: array[0..15, 0..1] of LongInt;
  iZeroLevel, RangeSpan: Integer;
  bProgrammaticChange: Boolean;
  bCountEditing: array[0..1] of Boolean;
  bCleared: Boolean;
  ColorScheme: (csLightOnDark, csDarkOnLight);

procedure TMainForm.FormCreate(Sender: TObject);
var
  Status, PID: LongWord;
  DetectForm: TDetectForm;

begin
  WriteAllButton.Align := alClient;
  LoadDefaultFont(Font);

  CalCountEdit[0] := CalCountEditLo;
  CalCountEdit[1] := CalCountEditHi;

  mVErrorEdit[0] := mVErrorEditLo;
  mVErrorEdit[1] := mVErrorEditHi;

  mVErrorButton[0] := mVErrorButtonLo;
  mVErrorButton[1] := mVErrorButtonHi;

  CalFN := GetTempDir + 'Cal.txt';

  DeviceIndex := ResolveDeviceIndex(diOnly);
  Status := QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
  if (Status <> ERROR_SUCCESS) or (PID < $8070) or (PID > $807F) then begin
    DetectForm := TDetectForm.Create(Self);
    if DetectForm.ShowModal = mrOK then begin
      DeviceIndex := DetectForm.DeviceIndex;
      DetectForm.Free;
      QueryDeviceInfo(DeviceIndex, @PID, nil, nil, nil, nil);
    end
    else begin
      DetectForm.Free;
      Application.Terminate;
      Exit;
    end;
  end;

  case PID of
    $8070, $8071, $8078, $8079: Channels := 16;
    $8072, $8073, $807A, $807B: Channels := 12;
    $8074, $8075, $807C, $807D: Channels := 8;
    $8076, $8077, $807E, $807F: Channels := 4;
    else Channels := 0;
  end;

  BlurbMemo.Lines.Text := 'Select a DAC. Select a level to set that DAC to its minimum, middle, or maximum value, and measure the error.'
            + #13#10#13#10'The max is affected only by the high cal point, the min only by the low cal point, and the mid by both. Adjust the high and low cal points:'
            +     #13#10' A few counts at a time with the  buttons.'
            +     #13#10' By typing new count values.'
            +     #13#10' By typing the error in mV and clicking the Correct mV Error button. For example, if +5V comes out as +4.985V, then type in "-15" and click Correct mV Error.' 
            + #13#10#13#10'Repeat for all levels of the selected DAC, and repeat for all DACs listed.'
            + #13#10#13#10'Click "Write All Cal To Device", and wait(about 15 seconds per DAC).'
  ;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  OnShow := nil;

  Application.OnIdle := IdleInit;
end;

procedure TMainForm.IdleInit(Sender: TObject; var Done: Boolean);
const
  ButtonW = 96;
  ButtonH = 24;
  CheckH = 16;
  MemoW = 192;
  DefLoCal = $0248;
  DefHiCal = $FE31;

var
  ChildForm: TForm;
  ThisButton: TButton;
  ChCheck: array of TCheckBox;
  ThisMemo: TMemo;
  MR: TModalResult;
  iChannel, I: Integer;

  procedure SetBipolar;
  var
    iDAC: Integer;

  begin
    for iDAC := 0 to 15 do DACDirect(DeviceIndex, iDAC, $8000);
    DACSetBoardRange(DeviceIndex, 1);
    iZeroLevel := 1;
  end;

  procedure SetUnipolar;
  var
    iDAC: Integer;

  begin
    for iDAC := 0 to 15 do DACDirect(DeviceIndex, iDAC, $0000);
    DACSetBoardRange(DeviceIndex, 0);
    iZeroLevel := 2;
  end;

begin
  Application.OnIdle := nil;

  ChildForm := TForm.CreateNew(Self);
  ChildForm.Position := poMainFormCenter;
  ChildForm.Caption := 'Device Range';
  ChildForm.ClientWidth := ButtonW + 16;
  ChildForm.ClientHeight := 8 + (ButtonH+8)*4;

  ThisButton := TButton.Create(ChildForm);
  ThisButton.SetBounds(8, 8, ButtonW, ButtonH);
  ThisButton.Caption := '10V';
  ThisButton.ModalResult := 4;
  ThisButton.Parent := ChildForm;

  ThisButton := TButton.Create(ChildForm);
  ThisButton.SetBounds(8, 8 + (ButtonH+8), ButtonW, ButtonH);
  ThisButton.Caption := '0-10V';
  ThisButton.ModalResult := 5;
  ThisButton.Parent := ChildForm;

  ThisButton := TButton.Create(ChildForm);
  ThisButton.SetBounds(8, 8 + (ButtonH+8)*2, ButtonW, ButtonH);
  ThisButton.Caption := '5V';
  ThisButton.ModalResult := 6;
  ThisButton.Parent := ChildForm;

  ThisButton := TButton.Create(ChildForm);
  ThisButton.SetBounds(8, 8 + (ButtonH+8)*3, ButtonW, ButtonH);
  ThisButton.Caption := '0-5V';
  ThisButton.ModalResult := 7;
  ThisButton.Parent := ChildForm;

  MR := ChildForm.ShowModal;
  ChildForm.Free;

  case MR of
    4: begin
      SetBipolar;
      RangeSpan := 20;
      LevelRadios.Items[0] := 'Max (+10V)';
      LevelRadios.Items[1] := 'Mid (0V)';
      LevelRadios.Items[2] := 'Min (-10V)';
    end;
    5: begin
      SetUnipolar;
      RangeSpan := 10;
      LevelRadios.Items[0] := 'Max (+10V)';
      LevelRadios.Items[1] := 'Mid (+5V)';
      LevelRadios.Items[2] := 'Min (0V)';
    end;
    6: begin
      SetBipolar;
      RangeSpan := 10;
      LevelRadios.Items[0] := 'Max (+5V)';
      LevelRadios.Items[1] := 'Mid (0V)';
      LevelRadios.Items[2] := 'Min (-5V)';
    end;
    7: begin
      SetUnipolar;
      RangeSpan := 5;
      LevelRadios.Items[0] := 'Max (+5V)';
      LevelRadios.Items[1] := 'Mid (+2.5V)';
      LevelRadios.Items[2] := 'Min (0V)';
    end;
    else begin
      Application.Terminate;
      Exit;
    end;
  end;

  ChildForm := TForm.CreateNew(Self);
  ChildForm.Position := poMainFormCenter;
  ChildForm.Caption := 'Channels To Calibrate';
  ChildForm.ClientWidth := 8 + (ButtonW+8) + (MemoW+8);
  ChildForm.ClientHeight := 8 + (CheckH+8)*Channels + (ButtonH+8)*3;

  SetLength(ChCheck, Channels);
  for I := 0 to Channels-1 do begin
    ChCheck[I] := TCheckBox.Create(ChildForm);
    ChCheck[I].SetBounds(8, 8 + (CheckH+8)*I, ButtonW, CheckH);
    if I < 10 then
      ChCheck[I].Caption := 'Ch &' + IntToStr(I)
    else
      ChCheck[I].Caption := 'Ch ' + IntToStr(I) + ' (&' + NybbleToHex[I] + ')'
    ;
    ChCheck[I].Parent := ChildForm;
  end;

  ThisButton := TButton.Create(ChildForm);
  ThisButton.SetBounds(8, 8 + (CheckH+8)*Channels, ButtonW, ButtonH);
  ThisButton.Caption := 'Check All';
  ThisButton.OnClick := CheckAllButtonClick;
  ThisButton.Parent := ChildForm;

  ThisButton := TButton.Create(ChildForm);
  ThisButton.SetBounds(8, 8 + (CheckH+8)*Channels + (ButtonH+8), ButtonW, ButtonH);
  ThisButton.Caption := 'Uncheck All';
  ThisButton.OnClick := UncheckAllButtonClick;
  ThisButton.Parent := ChildForm;

  ThisButton := TButton.Create(ChildForm);
  ThisButton.SetBounds(8, 8 + (CheckH+8)*Channels + (ButtonH+8)*2, ButtonW, ButtonH);
  ThisButton.Caption := 'OK';
  ThisButton.ModalResult := mrOk;
  ThisButton.Default := True;
  ThisButton.Parent := ChildForm;

  ThisMemo := TMemo.Create(ChildForm);
  ThisMemo.SetBounds(8 + (ButtonW+8), 8, MemoW, ChildForm.ClientHeight-16);
  ThisMemo.Anchors := [aktop, akBottom, akLeft, akRight];
  ThisMemo.ScrollBars := ssVertical;
  ThisMemo.Parent := ChildForm;
  ThisMemo.Lines.Text := 'You can check only one channel this time, and re-run the program to calibrate each channel, or you can check any/all channels to calibrate them all at once.'
           + #13#10#13#10'This program will remove the calibration on the selected channels, in order to replace it with a fresh calibration. If the calibration process isn''t completed, then the channels will be left uncalibrated.'
  ;

  ChMask := 0;
  if ChildForm.ShowModal = mrOk then
    for I := 0 to Channels-1 do
      if ChCheck[I].Checked then begin
        ChMask := ChMask or (1 shl I);
        DACRadios.Items.Add(IntToStr(I));
      end
  ;
  ChildForm.Free;

  if ChMask = 0 then begin
    Application.Terminate;
    Exit;
  end;

  bCleared := True;
  for I := 0 to DACRadios.Items.Count-1 do begin
    iChannel := StrToInt(DACRadios.Items[I]);

    CalTar[iChannel, 0] := _DACCalRead(DeviceIndex, iChannel, $0000, DefLoCal);
    CalTar[iChannel, 1] := _DACCalRead(DeviceIndex, iChannel, $FFFF, DefHiCal);

    if (CalTar[iChannel, 0] > $0FFF) or (CalTar[iChannel, 0] = $0000) then CalTar[iChannel, 0] := DefLoCal;
    if (CalTar[iChannel, 1] < $1000) or (CalTar[iChannel, 1] = $FFFF) then CalTar[iChannel, 1] := DefHiCal;

    WaitForm.GoWait((DACRadios.Items.Count - I) * Est1To1Time);
    DACSetChannelCal(DeviceIndex, iChannel, ':1TO1:');
    Sleep(1000);
  end;
  WaitForm.Hide;

  DACRadios.SetFocus;
  DACRadios.ItemIndex := -1;
  DACRadios.ItemIndex := 0;
end;

procedure TMainForm.CheckAllButtonClick(Sender: TObject);
var
  Cont: TWinControl;
  ThisChild: TControl;
  iChild: Integer;

begin
  Cont := (Sender as TControl).Parent;
  for iChild := 0 to Cont.ControlCount-1 do begin
    ThisChild := Cont.Controls[iChild];
    if not (ThisChild is TCheckBox) then Continue;

    (ThisChild as TCheckBox).Checked := True;
  end;
end;

procedure TMainForm.UncheckAllButtonClick(Sender: TObject);
var
  Cont: TWinControl;
  ThisChild: TControl;
  iChild: Integer;

begin
  Cont := (Sender as TControl).Parent;
  for iChild := 0 to Cont.ControlCount-1 do begin
    ThisChild := Cont.Controls[iChild];
    if not (ThisChild is TCheckBox) then Continue;

    (ThisChild as TCheckBox).Checked := False;
  end;
end;

procedure TMainForm.DACRadiosClick(Sender: TObject);
var
  iDAC: Integer;

begin
  iDAC := DACRadios.ItemIndex;
  if iDAC < 0 then Exit;

  LevelRadios.ItemIndex := -1;
  LevelRadios.ItemIndex := iZeroLevel;
end;

procedure TMainForm.LevelRadiosClick(Sender: TObject);
var
  iDAC, iLevel: Integer;

begin
  iDAC := DACRadios.ItemIndex;
  if iDAC < 0 then Exit;

  iDAC := StrToInt(DACRadios.Items[iDAC]);

  iLevel := LevelRadios.ItemIndex;
  if iLevel < 0 then Exit;

  CalGroupHi.Visible := iLevel <> 2;
  CalGroupLo.Visible := iLevel <> 0;

  UpdateOutput(iDAC);
end;

procedure TMainForm.WriteOneButtonClick(Sender: TObject);
var
  iDAC: Integer;

begin
  iDAC := StrToInt(DACRadios.Items[DACRadios.ItemIndex]);
  BuildCalFile(iDAC);

  WaitForm.GoWait(EstWriteTime);
  DACSetChannelCal(DeviceIndex, iDAC, PChar(CalFN));
  Sleep(1000);
  WaitForm.Hide;
end;

procedure TMainForm.WriteAllButtonClick(Sender: TObject);
var
  iDAC, I, L: Integer;
  TextStr: String;

begin
  TextStr := '';
  L := DACRadios.Items.Count;
  for I := 0 to L-1 do begin
    iDAC := StrToInt(DACRadios.Items[I]);

    BuildCalFile(iDAC);

    WaitForm.GoWait((Channels - I) * EstWriteTime);
    DACSetChannelCal(DeviceIndex, iDAC, PChar(CalFN));
    Sleep(1000);

    if I <> 0 then TextStr := TextStr + ', ';
    TextStr := TextStr + IntToStr(iDAC);
  end;
  WaitForm.Hide;
  bCleared := False;

  TextStr := 'Cal written to ' + IntToStr(L) + ' DAC' + PluralS(L) + ':'#13#10 + TextStr;
  Application.MessageBox(@TextStr[1], 'Cal Written', MB_ICONINFORMATION);
  Close;
end;

procedure TMainForm.BuildCalFile(iDAC: Integer);
var
  I, J: Integer;
  LoCt, HiCt: Integer;
  F: Double;
  Buf: AnsiString;

begin
  LoCt := CalTar[iDAC, 0];
  HiCt := CalTar[iDAC, 1];

  SetLength(Buf, $70000);

  for I := 0 to $FFFF do begin
    F := (I - 0) / $FFFF;
    F := F * (HiCt - LoCt) + LoCt;
    J := Round(F);
    //Fil.WriteStr('$' + IntToHex(J, 4) + #13#10);
    Buf[I * 7 + 7] := #10;
    Buf[I * 7 + 6] := #13;
    Buf[I * 7 + 5] := NybbleToHex[J and $F];
    J := J shr 4;
    Buf[I * 7 + 4] := NybbleToHex[J and $F];
    J := J shr 4;
    Buf[I * 7 + 3] := NybbleToHex[J and $F];
    J := J shr 4;
    Buf[I * 7 + 2] := NybbleToHex[J and $F];
    Buf[I * 7 + 1] := '$';
  end;

  DumpStringToFile(Buf, CalFN);
end;

procedure TMainForm.FormResize(Sender: TObject);
var
  CR: TRect;
  W, EdgeY: Integer;

begin
  CR := CalibratePanel.ClientRect;
  W := CR.Right - CR.Left;
  EdgeY := (CR.Top + CR.Bottom) div 2;

  CalGroupHi.SetBounds(CR.Left, CR.Top, W, EdgeY - CR.Top);
  CalGroupLo.SetBounds(CR.Left, EdgeY, W, CR.Bottom - EdgeY);
end;

procedure TMainForm.UpdateOutput(iDAC: Integer);
var
  iLevel: Integer;

begin
  iLevel := LevelRadios.ItemIndex;
  if iLevel < 0 then Exit;

  case iLevel of
    0: DACDirect(DeviceIndex, iDAC, CalTar[iDAC, 1]);
    1: DACDirect(DeviceIndex, iDAC, (CalTar[iDAC, 0] + CalTar[iDAC, 1]) div 2);
    2: DACDirect(DeviceIndex, iDAC, CalTar[iDAC, 0]);
  end;

  bProgrammaticChange := True;
  CalCountEditHi.Text := IntToHex(CalTar[iDAC, 1], 4);
  CalCountEditLo.Text := IntToHex(CalTar[iDAC, 0], 4);
  bProgrammaticChange := False;
end;

procedure TMainForm.mVErrorButtonClick(Sender: TObject);
var
  iDAC, iLevel: Integer;
  ErrStr: String;
  nTar: Integer;

begin
  iDAC := DACRadios.ItemIndex;
  if iDAC < 0 then Exit;

  iLevel := (Sender as TButton).Parent.Tag;
  if iLevel < 0 then Exit;

  ErrStr := mVErrorEdit[iLevel].Text;
  mVErrorEdit[iLevel].Text := '0';

  nTar := CalTar[iDAC, iLevel] - Round($10000 * StrToFloat(ErrStr) / (RangeSpan * 1000));

  nTar := Min($FFFF, Max($0000, nTar));
  CalTar[iDAC, iLevel] := nTar;
  UpdateOutput(iDAC);
end;

procedure TMainForm.AdjustButtonClick(Sender: TObject);
var
  iDAC, iLevel: Integer;
  nTar: Integer;

begin
  iDAC := DACRadios.ItemIndex;
  if iDAC < 0 then Exit;

  iLevel := (Sender as TButton).Parent.Tag;
  if iLevel < 0 then Exit;

  nTar := CalTar[iDAC, iLevel] + (Sender as TButton).Tag;

  nTar := Min($FFFF, Max($0000, nTar));
  CalTar[iDAC, iLevel] := nTar;
  UpdateOutput(iDAC);
end;

procedure TMainForm.CalCountEditChange(Sender: TObject);
var
  iDAC, iLevel: Integer;
  nTar: Integer;
  LightDiff: Integer;
  C: TColor;
  CB: array[0..4] of Byte absolute C;
  DarkC, LightC: TColor;

begin
  if bProgrammaticChange then Exit;

  iDAC := DACRadios.ItemIndex;
  if iDAC < 0 then Exit;

  iLevel := (Sender as TEdit).Parent.Tag;
  if iLevel < 0 then Exit;

  if not bCountEditing[iLevel] then begin
    C := ColorToRGB(clWindowText);
    LightDiff := CB[0] + CB[1] + CB[2];
    C := ColorToRGB(clWindow);
    LightDiff := LightDiff - (CB[0] + CB[1] + CB[2]);

    if LightDiff < 0 then
      ColorScheme := csDarkOnLight
    else
      ColorScheme := csLightOnDark
    ;

    bCountEditing[iLevel] := True;
  end;

  DarkC := $000040;
  LightC := $C0C0FF;
  try
    nTar := StrToInt('$' + CalCountEdit[iLevel].Text);
    if (nTar < $0000) or (nTar > $FFFF) then begin
      DarkC := $002020;
      LightC := $00FFFF;
    end
    else begin
      DarkC := $400000;
      LightC := $FFC0C0;
    end;
  except
  end;

  if ColorScheme = csDarkOnLight then begin
    CalCountEdit[iLevel].Color := LightC;
    CalCountEdit[iLevel].Font.Color := DarkC;
  end
  else begin
    CalCountEdit[iLevel].Color := DarkC;
    CalCountEdit[iLevel].Font.Color := LightC;
  end;
end;

procedure TMainForm.CalCountEditExit(Sender: TObject);
var
  iDAC, iLevel: Integer;
  nTar: Integer;

begin
  iDAC := DACRadios.ItemIndex;
  if iDAC < 0 then Exit;

  iLevel := (Sender as TEdit).Parent.Tag;
  if iLevel < 0 then Exit;

  if not bCountEditing[iLevel] then Exit;

  bCountEditing[iLevel] := False;
  CalCountEdit[iLevel].Color := clWindow;
  CalCountEdit[iLevel].ParentFont := True;

  try
    nTar := StrToInt('$' + CalCountEdit[iLevel].Text);
  except
    bProgrammaticChange := True;
    CalCountEdit[iLevel].Text := IntToHex(CalTar[iDAC, iLevel], 4);
    bProgrammaticChange := False;
    Exit;
  end;

  nTar := Min($FFFF, Max($0000, nTar));
  CalTar[iDAC, iLevel] := nTar;
  UpdateOutput(iDAC);
end;

procedure TMainForm.CalCountEditKeyPress(Sender: TObject; var Key: Char);
var
  iDAC, iLevel: Integer;
  nTar: Integer;

begin
  if Key <> #13 then Exit;

  Key := #0;

  iDAC := DACRadios.ItemIndex;
  if iDAC < 0 then Exit;

  iLevel := (Sender as TEdit).Parent.Tag;
  if iLevel < 0 then Exit;

  if not bCountEditing[iLevel] then Exit;

  bCountEditing[iLevel] := False;
  CalCountEdit[iLevel].Color := clWindow;
  CalCountEdit[iLevel].ParentFont := True;

  try
    nTar := StrToInt('$' + CalCountEdit[iLevel].Text);
  except
    bProgrammaticChange := True;
    CalCountEdit[iLevel].Text := IntToHex(CalTar[iDAC, iLevel], 4);
    bProgrammaticChange := False;
    raise;
  end;

  nTar := Min($FFFF, Max($0000, nTar));
  CalTar[iDAC, iLevel] := nTar;
  UpdateOutput(iDAC);
end;

procedure TMainForm.mVErrorEditKeyPress(Sender: TObject; var Key: Char);
var
  iLevel: Integer;

begin
  if Key <> #13 then Exit;

  Key := #0;

  iLevel := (Sender as TEdit).Parent.Tag;
  if iLevel < 0 then Exit;

  mVErrorButtonClick(mVErrorButton[iLevel]);
end;

end.
