unit TheseUtils;

interface

uses
  Windows,
  SysUtils, Classes, Graphics,
  AIOUSB;

const
  NybbleToHex: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');

const
  //Currently in quarter-seconds.
  EstEraseTime = 2;
  Est1To1Time  = 20;
  EstWriteTime = 54;

function PluralS(I: Integer): String;
  
procedure DumpStringToFile(S: AnsiString; Path: String);
function DumpFileToString(Path: String): AnsiString;

function GetTempDir: String;
procedure LoadDefaultFont(IntoFont: TFont);

function _FlashRead(DeviceIndex, StartAddress: LongWord; var DataSize: LongWord; pData: Pointer): LongWord;
function _DACCalRead(DeviceIndex, DACIndex: LongWord; CountIndex: Word; DefaultData: Word): Word;





implementation

function PluralS(I: Integer): String;
begin
  if I = 1 then
    Result := ''
  else
    Result := 's'
  ;
end;



procedure DumpStringToFile(S: AnsiString; Path: String);
var
  Fil: TFileStream;

begin
  Fil := TFileStream.Create(Path, fmCreate or fmShareDenyWrite);
  try
    Fil.Write(S[1], Length(S));
  finally
    Fil.Free;
  end;
end;

function DumpFileToString(Path: String): AnsiString;
var
  Fil: TFileStream;
  L: Integer;

begin
  Fil := TFileStream.Create(Path, fmOpenRead or fmShareDenyNone);
  try
    L := Fil.Size;
    SetString(Result, nil, L);
    Fil.Read(Result[1], L);
  finally
    Fil.Free;
  end;
end;



function GetTempDir: String;
var
  L: Integer;

begin
  SetString(Result, nil, MAX_PATH);
  L := GetTempPath(Length(Result), @Result[1]);
  SetLength(Result, L);
  Result := IncludeTrailingBackslash(Result);
end;

procedure LoadDefaultFont(IntoFont: TFont);
var
  FontSize: Integer;
  NCM: TNonClientMetrics;

begin
  NCM.cbSize := SizeOf(NCM);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(NCM), @NCM, 0) then begin
    FontSize := Round(-3/4 * NCM.lfMessageFont.lfHeight);
    IntoFont.Name := NCM.lfMessageFont.lfFaceName;
  end
  else begin
    FontSize := Abs(IntoFont.Size);
    IntoFont.Name := 'Tahoma';
  end;
  IntoFont.PixelsPerInch := 72;
  IntoFont.Size := FontSize;
end;



function _FlashRead(DeviceIndex, StartAddress: LongWord; var DataSize: LongWord; pData: Pointer): LongWord;
const
  AUR_FLASH_READWRITE     = $B5;

begin
  Result := GenericVendorRead(DeviceIndex, AUR_FLASH_READWRITE, StartAddress shr 1, StartAddress shr 17, DataSize, pData);
end;

function _DACCalRead(DeviceIndex, DACIndex: LongWord; CountIndex: Word; DefaultData: Word): Word;
var
  CalDataSize: LongWord;
  StartAddress: LongWord;

begin
  Result := DefaultData;
  CalDataSize := SizeOf(Result);
  StartAddress := (DACIndex * $10000 + CountIndex) * 2;
  SetLastError(_FlashRead(DeviceIndex, StartAddress, CalDataSize, @Result));
end;

end.
